import { __decorate, __param } from 'vscode/external/tslib/tslib.es6.js';
import { localizeWithPath, localize2WithPath } from 'vscode/vscode/vs/nls';
import { Disposable } from 'vscode/vscode/vs/base/common/lifecycle';
import { Registry } from 'vscode/vscode/vs/platform/registry/common/platform';
import { registerAction2, MenuRegistry, MenuId } from 'vscode/vscode/vs/platform/actions/common/actions';
import { ProblemMatcherRegistry } from '../common/problemMatcher.js';
import { IProgressService } from 'vscode/vscode/vs/platform/progress/common/progress.service';
import { Extensions as Extensions$3 } from 'vscode/vscode/vs/platform/jsonschemas/common/jsonContributionRegistry';
import { IStatusbarService } from 'vscode/vscode/vs/workbench/services/statusbar/browser/statusbar.service';
import { Extensions as Extensions$1 } from 'vscode/vscode/vs/workbench/services/output/common/output';
import { TASKS_CATEGORY, TaskGroup, TASK_RUNNING_STATE } from 'vscode/vscode/vs/workbench/contrib/tasks/common/tasks';
import { TaskExecutionSupportedContext, TaskCommandsRegistered } from 'vscode/vscode/vs/workbench/contrib/tasks/common/taskService';
import { ITaskService } from 'vscode/vscode/vs/workbench/contrib/tasks/common/taskService.service';
import { Extensions } from 'vscode/vscode/vs/workbench/common/contributions';
import { RunAutomaticTasks, ManageAutomaticTaskRunning } from './runAutomaticTasks.js';
import { KeybindingsRegistry } from 'vscode/vscode/vs/platform/keybinding/common/keybindingsRegistry';
import schema$1 from '../common/jsonSchema_v1.js';
import schema$2, { updateProblemMatchers, updateTaskDefinitions } from '../common/jsonSchema_v2.js';
import { ConfigureTaskAction, AbstractTaskService } from './abstractTaskService.js';
import { tasksSchemaId } from 'vscode/vscode/vs/workbench/services/configuration/common/configuration';
import { Extensions as Extensions$4 } from 'vscode/vscode/vs/platform/configuration/common/configurationRegistry';
import { WorkbenchStateContext } from 'vscode/vscode/vs/workbench/common/contextkeys';
import { Extensions as Extensions$2 } from 'vscode/vscode/vs/platform/quickinput/common/quickAccess';
import { TasksQuickAccessProvider } from './tasksQuickAccess.js';
import { ContextKeyExpr } from 'vscode/vscode/vs/platform/contextkey/common/contextkey';
import { TaskDefinitionRegistry } from 'vscode/vscode/vs/workbench/contrib/tasks/common/taskDefinitionRegistry';
import { isString } from 'vscode/vscode/vs/base/common/types';
import { promiseWithResolvers } from 'vscode/vscode/vs/base/common/async';

const _moduleId = "vs/workbench/contrib/tasks/browser/task.contribution";
const workbenchRegistry = ( (Registry.as(Extensions.Workbench)));
workbenchRegistry.registerWorkbenchContribution(RunAutomaticTasks, 4 );
registerAction2(ManageAutomaticTaskRunning);
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: ManageAutomaticTaskRunning.ID,
        title: ManageAutomaticTaskRunning.LABEL,
        category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
let TaskStatusBarContributions = class TaskStatusBarContributions extends Disposable {
    constructor(_taskService, _statusbarService, _progressService) {
        super();
        this._taskService = _taskService;
        this._statusbarService = _statusbarService;
        this._progressService = _progressService;
        this._activeTasksCount = 0;
        this._registerListeners();
    }
    _registerListeners() {
        let promise = undefined;
        let resolve;
        this._register(this._taskService.onDidStateChange(event => {
            if (event.kind === "changed" ) {
                this._updateRunningTasksStatus();
            }
            if (!this._ignoreEventForUpdateRunningTasksCount(event)) {
                switch (event.kind) {
                    case "active" :
                        this._activeTasksCount++;
                        if (this._activeTasksCount === 1) {
                            if (!promise) {
                                ({ promise, resolve } = promiseWithResolvers());
                            }
                        }
                        break;
                    case "inactive" :
                        if (this._activeTasksCount > 0) {
                            this._activeTasksCount--;
                            if (this._activeTasksCount === 0) {
                                if (promise && resolve) {
                                    resolve();
                                }
                            }
                        }
                        break;
                    case "terminated" :
                        if (this._activeTasksCount !== 0) {
                            this._activeTasksCount = 0;
                            if (promise && resolve) {
                                resolve();
                            }
                        }
                        break;
                }
            }
            if (promise && ((event.kind === "active") ) && (this._activeTasksCount === 1)) {
                this._progressService.withProgress({ location: 10 , command: 'workbench.action.tasks.showTasks', type: 'loading' }, progress => {
                    progress.report({ message: ( localizeWithPath(_moduleId, 0, 'Building...')) });
                    return promise;
                }).then(() => {
                    promise = undefined;
                });
            }
        }));
    }
    async _updateRunningTasksStatus() {
        const tasks = await this._taskService.getActiveTasks();
        if (tasks.length === 0) {
            if (this._runningTasksStatusItem) {
                this._runningTasksStatusItem.dispose();
                this._runningTasksStatusItem = undefined;
            }
        }
        else {
            const itemProps = {
                name: ( localizeWithPath(_moduleId, 1, "Running Tasks")),
                text: `$(tools) ${tasks.length}`,
                ariaLabel: ( localizeWithPath(_moduleId, 2, "{0} running tasks", tasks.length)),
                tooltip: ( localizeWithPath(_moduleId, 3, "Show Running Tasks")),
                command: 'workbench.action.tasks.showTasks',
            };
            if (!this._runningTasksStatusItem) {
                this._runningTasksStatusItem = this._statusbarService.addEntry(itemProps, 'status.runningTasks', 0 , 49 );
            }
            else {
                this._runningTasksStatusItem.update(itemProps);
            }
        }
    }
    _ignoreEventForUpdateRunningTasksCount(event) {
        if (!this._taskService.inTerminal() || event.kind === "changed" ) {
            return false;
        }
        if ((isString(event.group) ? event.group : event.group?._id) !== TaskGroup.Build._id) {
            return true;
        }
        return event.__task.configurationProperties.problemMatchers === undefined || event.__task.configurationProperties.problemMatchers.length === 0;
    }
};
TaskStatusBarContributions = ( (__decorate([
    ( (__param(0, ITaskService))),
    ( (__param(1, IStatusbarService))),
    ( (__param(2, IProgressService)))
], TaskStatusBarContributions)));
workbenchRegistry.registerWorkbenchContribution(TaskStatusBarContributions, 3 );
MenuRegistry.appendMenuItem(MenuId.MenubarTerminalMenu, {
    group: "3_run" ,
    command: {
        id: 'workbench.action.tasks.runTask',
        title: ( localizeWithPath(_moduleId, 4, "&&Run Task..."))
    },
    order: 1,
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.MenubarTerminalMenu, {
    group: "3_run" ,
    command: {
        id: 'workbench.action.tasks.build',
        title: ( localizeWithPath(_moduleId, 5, "Run &&Build Task..."))
    },
    order: 2,
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.MenubarTerminalMenu, {
    group: "5_manage" ,
    command: {
        precondition: TASK_RUNNING_STATE,
        id: 'workbench.action.tasks.showTasks',
        title: ( localizeWithPath(_moduleId, 6, "Show Runnin&&g Tasks..."))
    },
    order: 1,
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.MenubarTerminalMenu, {
    group: "5_manage" ,
    command: {
        precondition: TASK_RUNNING_STATE,
        id: 'workbench.action.tasks.restartTask',
        title: ( localizeWithPath(_moduleId, 7, "R&&estart Running Task..."))
    },
    order: 2,
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.MenubarTerminalMenu, {
    group: "5_manage" ,
    command: {
        precondition: TASK_RUNNING_STATE,
        id: 'workbench.action.tasks.terminate',
        title: ( localizeWithPath(_moduleId, 8, "&&Terminate Task..."))
    },
    order: 3,
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.MenubarTerminalMenu, {
    group: "7_configure" ,
    command: {
        id: 'workbench.action.tasks.configureTaskRunner',
        title: ( localizeWithPath(_moduleId, 9, "&&Configure Tasks..."))
    },
    order: 1,
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.MenubarTerminalMenu, {
    group: "7_configure" ,
    command: {
        id: 'workbench.action.tasks.configureDefaultBuildTask',
        title: ( localizeWithPath(_moduleId, 10, "Configure De&&fault Build Task..."))
    },
    order: 2,
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.openWorkspaceFileTasks',
        title: ( localize2WithPath(_moduleId, 11, "Open Workspace Tasks")),
        category: TASKS_CATEGORY
    },
    when: ( (ContextKeyExpr.and(
         (WorkbenchStateContext.isEqualTo('workspace')),
        TaskExecutionSupportedContext
    )))
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: ConfigureTaskAction.ID,
        title: ConfigureTaskAction.TEXT,
        category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.showLog',
        title: ( localize2WithPath(_moduleId, 12, "Show Task Log")),
        category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.runTask',
        title: ( localize2WithPath(_moduleId, 13, "Run Task")),
        category: TASKS_CATEGORY
    }
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.reRunTask',
        title: ( localize2WithPath(_moduleId, 14, "Rerun Last Task")),
        category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.restartTask',
        title: ( localize2WithPath(_moduleId, 15, "Restart Running Task")),
        category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.showTasks',
        title: ( localize2WithPath(_moduleId, 16, "Show Running Tasks")),
        category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.terminate',
        title: ( localize2WithPath(_moduleId, 17, "Terminate Task")),
        category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.build',
        title: ( localize2WithPath(_moduleId, 18, "Run Build Task")),
        category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.test',
        title: ( localize2WithPath(_moduleId, 19, "Run Test Task")),
        category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.configureDefaultBuildTask',
        title: ( localize2WithPath(_moduleId, 20, "Configure Default Build Task")),
        category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.configureDefaultTestTask',
        title: ( localize2WithPath(_moduleId, 21, "Configure Default Test Task")),
        category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.openUserTasks',
        title: ( localize2WithPath(_moduleId, 22, "Open User Tasks")), category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
class UserTasksGlobalActionContribution extends Disposable {
    constructor() {
        super();
        this.registerActions();
    }
    registerActions() {
        const id = 'workbench.action.tasks.openUserTasks';
        const title = ( localizeWithPath(_moduleId, 23, "User Tasks"));
        this._register(MenuRegistry.appendMenuItem(MenuId.GlobalActivity, {
            command: {
                id,
                title
            },
            when: TaskExecutionSupportedContext,
            group: '2_configuration',
            order: 6
        }));
        this._register(MenuRegistry.appendMenuItem(MenuId.MenubarPreferencesMenu, {
            command: {
                id,
                title
            },
            when: TaskExecutionSupportedContext,
            group: '2_configuration',
            order: 6
        }));
    }
}
workbenchRegistry.registerWorkbenchContribution(UserTasksGlobalActionContribution, 3 );
KeybindingsRegistry.registerKeybindingRule({
    id: 'workbench.action.tasks.build',
    weight: 200 ,
    when: TaskCommandsRegistered,
    primary: 2048  | 1024  | 32
});
const outputChannelRegistry = ( (Registry.as(Extensions$1.OutputChannels)));
outputChannelRegistry.registerChannel({ id: AbstractTaskService.OutputChannelId, label: AbstractTaskService.OutputChannelLabel, log: false });
const quickAccessRegistry = (( (Registry.as(Extensions$2.Quickaccess))));
const tasksPickerContextKey = 'inTasksPicker';
quickAccessRegistry.registerQuickAccessProvider({
    ctor: TasksQuickAccessProvider,
    prefix: TasksQuickAccessProvider.PREFIX,
    contextKey: tasksPickerContextKey,
    placeholder: ( localizeWithPath(_moduleId, 24, "Type the name of a task to run.")),
    helpEntries: [{ description: ( localizeWithPath(_moduleId, 25, "Run Task")), commandCenterOrder: 60 }]
});
const schema = {
    id: tasksSchemaId,
    description: 'Task definition file',
    type: 'object',
    allowTrailingCommas: true,
    allowComments: true,
    default: {
        version: '2.0.0',
        tasks: [
            {
                label: 'My Task',
                command: 'echo hello',
                type: 'shell',
                args: [],
                problemMatcher: ['$tsc'],
                presentation: {
                    reveal: 'always'
                },
                group: 'build'
            }
        ]
    }
};
schema.definitions = {
    ...schema$1.definitions,
    ...schema$2.definitions,
};
schema.oneOf = [...(schema$2.oneOf || []), ...(schema$1.oneOf || [])];
const jsonRegistry = ( (Registry.as(Extensions$3.JSONContribution)));
jsonRegistry.registerSchema(tasksSchemaId, schema);
ProblemMatcherRegistry.onMatcherChanged(() => {
    updateProblemMatchers();
    jsonRegistry.notifySchemaChanged(tasksSchemaId);
});
TaskDefinitionRegistry.onDefinitionsChanged(() => {
    updateTaskDefinitions();
    jsonRegistry.notifySchemaChanged(tasksSchemaId);
});
const configurationRegistry = ( (Registry.as(Extensions$4.Configuration)));
configurationRegistry.registerConfiguration({
    id: 'task',
    order: 100,
    title: ( localizeWithPath(_moduleId, 26, "Tasks")),
    type: 'object',
    properties: {
        ["task.problemMatchers.neverPrompt" ]: {
            markdownDescription: ( localizeWithPath(
                _moduleId,
                27,
                "Configures whether to show the problem matcher prompt when running a task. Set to `true` to never prompt, or use a dictionary of task types to turn off prompting only for specific task types."
            )),
            'oneOf': [
                {
                    type: 'boolean',
                    markdownDescription: ( localizeWithPath(_moduleId, 28, 'Sets problem matcher prompting behavior for all tasks.'))
                },
                {
                    type: 'object',
                    patternProperties: {
                        '.*': {
                            type: 'boolean'
                        }
                    },
                    markdownDescription: ( localizeWithPath(
                        _moduleId,
                        29,
                        'An object containing task type-boolean pairs to never prompt for problem matchers on.'
                    )),
                    default: {
                        'shell': true
                    }
                }
            ],
            default: false
        },
        ["task.autoDetect" ]: {
            markdownDescription: ( localizeWithPath(
                _moduleId,
                30,
                "Controls enablement of `provideTasks` for all task provider extension. If the Tasks: Run Task command is slow, disabling auto detect for task providers may help. Individual extensions may also provide settings that disable auto detection."
            )),
            type: 'string',
            enum: ['on', 'off'],
            default: 'on'
        },
        ["task.slowProviderWarning" ]: {
            markdownDescription: ( localizeWithPath(
                _moduleId,
                31,
                "Configures whether a warning is shown when a provider is slow"
            )),
            'oneOf': [
                {
                    type: 'boolean',
                    markdownDescription: ( localizeWithPath(_moduleId, 32, 'Sets the slow provider warning for all tasks.'))
                },
                {
                    type: 'array',
                    items: {
                        type: 'string',
                        markdownDescription: ( localizeWithPath(
                            _moduleId,
                            33,
                            'An array of task types to never show the slow provider warning.'
                        ))
                    }
                }
            ],
            default: true
        },
        ["task.quickOpen.history" ]: {
            markdownDescription: ( localizeWithPath(
                _moduleId,
                34,
                "Controls the number of recent items tracked in task quick open dialog."
            )),
            type: 'number',
            default: 30, minimum: 0, maximum: 30
        },
        ["task.quickOpen.detail" ]: {
            markdownDescription: ( localizeWithPath(
                _moduleId,
                35,
                "Controls whether to show the task detail for tasks that have a detail in task quick picks, such as Run Task."
            )),
            type: 'boolean',
            default: true
        },
        ["task.quickOpen.skip" ]: {
            type: 'boolean',
            description: ( localizeWithPath(
                _moduleId,
                36,
                "Controls whether the task quick pick is skipped when there is only one task to pick from."
            )),
            default: false
        },
        ["task.quickOpen.showAll" ]: {
            type: 'boolean',
            description: ( localizeWithPath(
                _moduleId,
                37,
                "Causes the Tasks: Run Task command to use the slower \"show all\" behavior instead of the faster two level picker where tasks are grouped by provider."
            )),
            default: false
        },
        ["task.allowAutomaticTasks" ]: {
            type: 'string',
            enum: ['on', 'off'],
            enumDescriptions: [
                ( localizeWithPath(_moduleId, 38, "Always")),
                ( localizeWithPath(_moduleId, 39, "Never")),
            ],
            description: ( localizeWithPath(
                _moduleId,
                40,
                "Enable automatic tasks - note that tasks won't run in an untrusted workspace."
            )),
            default: 'on',
            restricted: true
        },
        ["task.reconnection" ]: {
            type: 'boolean',
            description: ( localizeWithPath(
                _moduleId,
                41,
                "On window reload, reconnect to tasks that have problem matchers."
            )),
            default: true
        },
        ["task.saveBeforeRun" ]: {
            markdownDescription: ( localizeWithPath(_moduleId, 42, 'Save all dirty editors before running a task.')),
            type: 'string',
            enum: ['always', 'never', 'prompt'],
            enumDescriptions: [
                ( localizeWithPath(_moduleId, 43, 'Always saves all editors before running.')),
                ( localizeWithPath(_moduleId, 44, 'Never saves editors before running.')),
                ( localizeWithPath(_moduleId, 45, 'Prompts whether to save editors before running.')),
            ],
            default: 'always',
        },
        ["task.verboseLogging" ]: {
            type: 'boolean',
            description: ( localizeWithPath(_moduleId, 46, "Enable verbose logging for tasks.")),
            default: false
        },
    }
});

export { TaskStatusBarContributions };
